/* Copyright (c) 2015 Nordic Semiconductor. All Rights Reserved.
 *
 * The information contained herein is property of Nordic Semiconductor ASA.
 * Terms and conditions of usage are described in detail in NORDIC
 * SEMICONDUCTOR STANDARD SOFTWARE LICENSE AGREEMENT.
 *
 * Licensees are granted free, non-transferable use of the information. NO
 * WARRANTY of ANY KIND is provided. This heading must NOT be removed from
 * the file.
 *
 */

/** @file
 * @defgroup tw_sensor_example main.c
 * @{
 * @ingroup nrf_twi_example
 * @brief TWI Sensor Example main file.
 *
 * This file contains the source code for a sample application using TWI.
 *
 */

#include <stdio.h>
#include "boards.h"
#include "app_util_platform.h"
#include "app_uart.h"
#include "app_error.h"
#include "nrf_drv_twi.h"
#include "nrf_delay.h"

/*UART buffer size. */
#define UART_TX_BUF_SIZE 256
#define UART_RX_BUF_SIZE 1

/* Marco definitions*/
/* TWI instance for RTC. */
#define RTC_SCL_PIN             11    // SCL signal pin
#define RTC_SDA_PIN             12    // SDA signal pin
static const nrf_drv_twi_t m_twi_rtc = NRF_DRV_TWI_INSTANCE(0);

/* TWI instance for Biosensor. */
#define BIO_SCL_PIN             17    // SCL signal pin
#define BIO_SDA_PIN             16    // SDA signal pin
static const nrf_drv_twi_t m_twi_bio = NRF_DRV_TWI_INSTANCE(0);

/**
 * @brief TWI events handler for RTC.
 */
static void rtc_twi_handler(nrf_drv_twi_evt_t const * p_event, void * p_context)
{   
    switch(p_event->type)
    {
        case NRF_DRV_TWI_EVT_DONE:
            if (p_event->xfer_desc.type == NRF_DRV_TWI_XFER_TX) {
                m_xfer_tx_done = true;
            }
            if (p_event->xfer_desc.type == NRF_DRV_TWI_XFER_RX) {
                m_xfer_rx_done = true;
            }
            if (p_event->xfer_desc.type == NRF_DRV_TWI_XFER_TXRX) {
                m_xfer_txrx_done = true;
            }
            //m_xfer_done = true;
            break;
        default:
            break;        
    }   
}

/**
 * @brief TWI events handler for Biosensor.
 */
static void bio_twi_handler(nrf_drv_twi_evt_t const * p_event, void * p_context)
{   
    switch(p_event->type)
    {
        case NRF_DRV_TWI_EVT_DONE:
            if (p_event->xfer_desc.type == NRF_DRV_TWI_XFER_TX) {
                m_xfer_tx_done = true;
            }
            if (p_event->xfer_desc.type == NRF_DRV_TWI_XFER_RX) {
                m_xfer_rx_done = true;
            }
            if (p_event->xfer_desc.type == NRF_DRV_TWI_XFER_TXRX) {
                m_xfer_txrx_done = true;
            }
            //m_xfer_done = true;
            break;
        default:
            break;        
    }   
}

/**
 * @brief UART events handler.
 */
static void uart_events_handler(app_uart_evt_t * p_event)
{
    switch (p_event->evt_type)
    {
        case APP_UART_COMMUNICATION_ERROR:
            APP_ERROR_HANDLER(p_event->data.error_communication);
            break;

        case APP_UART_FIFO_ERROR:
            APP_ERROR_HANDLER(p_event->data.error_code);
            break;

        default:
            break;
    }
}

static void rtc_twi_init()
{
    ret_code_t err_code;
    
    const nrf_drv_twi_config_t twi_rtc_config = {
       .scl                = RTC_SCL_PIN,
       .sda                = RTC_SDA_PIN,
       .frequency          = NRF_TWI_FREQ_400K,
       .interrupt_priority = APP_IRQ_PRIORITY_HIGH
    };
    
    err_code = nrf_drv_twi_init(&m_twi_rtc, &twi_rtc_config, rtc_twi_handler, NULL);
    APP_ERROR_CHECK(err_code);

    nrf_drv_twi_enable(&m_twi_rtc);}

/* Biosensor initialization*/
static void bio_twi_init(void)
{
    ret_code_t err_code;
    
    const nrf_drv_twi_config_t twi_bio_config = {
       .scl                = BIO_SCL_PIN,
       .sda                = BIO_SDA_PIN,
       .frequency          = NRF_TWI_FREQ_400K,
       .interrupt_priority = APP_IRQ_PRIORITY_HIGH
    };
    
    err_code = nrf_drv_twi_init(&m_twi_bio, &twi_bio_config, bio_twi_handler, NULL);
    APP_ERROR_CHECK(err_code);

    nrf_drv_twi_enable(&m_twi_bio);
/**
 * @brief UART initialization.
 */
static void uart_config(void)
{
    uint32_t                     err_code;
    const app_uart_comm_params_t comm_params =
    {
        RX_PIN_NUMBER,
        TX_PIN_NUMBER,
        RTS_PIN_NUMBER,
        CTS_PIN_NUMBER,
        APP_UART_FLOW_CONTROL_DISABLED,
        false,
        UART_BAUDRATE_BAUDRATE_Baud38400
    };

    APP_UART_FIFO_INIT(&comm_params,
                       UART_RX_BUF_SIZE,
                       UART_TX_BUF_SIZE,
                       uart_events_handler,
                       APP_IRQ_PRIORITY_LOW,
                       err_code);

    APP_ERROR_CHECK(err_code);
}
/**
 * @brief UART initialization.
 */
void twi_init (void)
{
    //i2c_gpio_init();
    // Init I2C
    rtc_twi_init();
    bio_twi_init();
}

/**
 * @brief Function for main application entry.
 */
int main(void)
{
	//gpio_config();
    uart_config();
    twi_init();
}

/** @} */
